@@ -75,7 +75,7 @@ class TestIndex(TestBase):
 
     def _assert_fprogress(self, entries):
         self.assertEqual(len(entries), len(self._fprogress_map))
-        for path, call_count in self._fprogress_map.items():  # @UnusedVariable
+        for _path, call_count in self._fprogress_map.items():
             self.assertEqual(call_count, 2)
         # END for each item in progress map
         self._reset_progress()
@@ -201,7 +201,7 @@ class TestIndex(TestBase):
 
         # test BlobFilter
         prefix = 'lib/git'
-        for stage, blob in base_index.iter_blobs(BlobFilter([prefix])):  # @UnusedVariable
+        for _stage, blob in base_index.iter_blobs(BlobFilter([prefix])):
             assert blob.path.startswith(prefix)
 
         # writing a tree should fail with an unmerged index
@@ -772,7 +772,16 @@ class TestIndex(TestBase):
             orig_tree = commit.tree
             self.assertEqual(index.write_tree(), orig_tree)
         # END for each commit
-
+    
+    @with_rw_repo('HEAD', bare=False)
+    def test_index_single_addremove(self, rw_repo):
+        fp = osp.join(rw_repo.working_dir, 'testfile.txt')
+        with open(fp, 'w') as fs:
+            fs.write(u'content of testfile')
+        self._assert_entries(rw_repo.index.add(fp))
+        deleted_files = rw_repo.index.remove(fp)
+        assert deleted_files
+        
     def test_index_new(self):
         B = self.rorepo.tree("6d9b1f4f9fa8c9f030e3207e7deacc5d5f8bba4e")
         H = self.rorepo.tree("25dca42bac17d511b7e2ebdd9d1d679e7626db5f")
@@ -897,7 +906,7 @@ class TestIndex(TestBase):
         _make_hook(
             index.repo.git_dir,
             'commit-msg',
-            'echo -n " {}" >> "$1"'.format(from_hook_message)
+            'printf " {}" >> "$1"'.format(from_hook_message)
         )
         new_commit = index.commit(commit_message)
         self.assertEqual(new_commit.message, u"{} {}".format(commit_message, from_hook_message))
